/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.ow2.bonita.pvm.internal.wire.binding;

import org.ow2.bonita.pvm.internal.deploy.CheckVersion;
import org.ow2.bonita.pvm.internal.util.XmlUtil;
import org.ow2.bonita.pvm.internal.wire.Descriptor;
import org.ow2.bonita.pvm.internal.wire.descriptor.FalseDescriptor;
import org.ow2.bonita.pvm.internal.wire.descriptor.ObjectDescriptor;
import org.ow2.bonita.pvm.internal.wire.descriptor.TrueDescriptor;
import org.ow2.bonita.pvm.internal.xml.Parse;
import org.ow2.bonita.pvm.internal.xml.Parser;
import org.w3c.dom.Element;

/**
 * parses a descriptor creating a {@link CheckVersion version verification
 * deployer}.
 * 
 * See schema docs for more details.
 * 
 * @author Tom Baeyens
 */
public class CheckVersionBinding extends WireDescriptorBinding {

  public CheckVersionBinding() {
    super("check-version");
  }

  public Object parse(Element element, Parse parse, Parser parser) {
    ObjectDescriptor objectDescriptor = new ObjectDescriptor(CheckVersion.class
        .getName());

    Boolean assign = XmlUtil.attributeBoolean(element, "assign", false, parse,
        null);
    if (assign != null) {
      Descriptor valueDescriptor = null;
      if (assign) {
        valueDescriptor = new TrueDescriptor();
      } else {
        valueDescriptor = new FalseDescriptor();
      }
      objectDescriptor.addInjection("assign", valueDescriptor);
    }

    return objectDescriptor;
  }
}
