/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.ow2.bonita.pvm.test.base;

import junit.framework.AssertionFailedError;
import junit.framework.TestCase;

import org.ow2.bonita.pvm.env.Environment;
import org.ow2.bonita.pvm.env.EnvironmentFactory;
import org.ow2.bonita.pvm.env.PvmEnvironmentFactory;
import org.ow2.bonita.pvm.internal.log.Log;

public abstract class PVMTestCase extends TestCase {

  @Override
  public void setUp() throws Exception {
    LOG.info("=== starting " + getName() + " =============================");
  }

  @Override
  public void tearDown() throws Exception {
    LOG.info("=== ending " + getName() + " =============================\n");
  }

  public void assertTextPresent(final String expected, final String value) {
    if ((value == null) || (value.indexOf(expected) == -1)) {
      throw new AssertionFailedError("expected presence of '" + expected
          + "' but was '" + value + "'");
    }
  }

  public static Environment openEnvironment(final String xmlString) {
    final EnvironmentFactory environmentFactory = PvmEnvironmentFactory
        .parseXmlString(xmlString);
    return environmentFactory.openEnvironment();
  }

  @Override
  protected void runTest() throws Throwable {
    try {
      super.runTest();
    } catch (final AssertionFailedError e) {
      LOG.error("");
      LOG.error("ASSERTION FAILURE: " + e.getMessage());
      LOG.error("");
      throw e;
    } catch (final Throwable t) {
      t.printStackTrace();
      throw t;
    }
  }

  protected static final Log LOG = Log.getLog(PVMTestCase.class.getName());
}
